Package: arsenal
Title: An Arsenal of 'R' Functions for Large-Scale Statistical
        Summaries
Version: 3.7.1
Date: 2026-07-01
Authors@R: c(
    person("Ethan", "Heinzen", email = "heinzen.ethan@mayo.edu", role = c("aut", "cre")),
    person("Jason", "Sinnwell", role="aut"),
    person("Elizabeth", "Atkinson", role="aut"),
    person("Tina", "Gunderson", role="aut"),
    person("Gregory", "Dougherty", role="aut"),
    person("Patrick", "Votruba", role="ctb"),
    person("Ryan", "Lennon", role="ctb"),
    person("Andrew", "Hanson", role="ctb"),
    person("Krista", "Goergen", role="ctb"),
    person("Emily", "Lundt", role="ctb"),
    person("Brendan", "Broderick", role="ctb"),
    person("Maddie", "McCullough", role="art")
  )
Description: An Arsenal of 'R' functions for large-scale statistical summaries,
  which are streamlined to work within the latest reporting tools in 'R' and
  'RStudio' and which use formulas and versatile summary statistics for summary
  tables and models. The primary functions include tableby(), a Table-1-like
  summary of multiple variable types 'by' the levels of one or more categorical
  variables; paired(), a Table-1-like summary of multiple variable types paired across
  two time points; modelsum(), which performs simple model fits on one or more endpoints
  for many variables (univariate or adjusted for covariates);
  freqlist(), a powerful frequency table across many categorical variables;
  comparedf(), a function for comparing data.frames; and
  write2(), a function to output tables to a document.
Suggests: broom (>= 0.7.1), magrittr, rmarkdown, testthat, xtable,
        pander, survival (>= 2.43-1), coin, pROC, MASS, splines, rpart,
        yaml, stddiff, geepack
Depends: R (>= 3.4.0), stats (>= 3.4.0)
Imports: knitr (>= 1.29), utils (>= 3.4.0), glue
URL: https://github.com/mayoverse/arsenal,
        https://cran.r-project.org/package=arsenal,
        https://mayoverse.github.io/arsenal/
BugReports: https://github.com/mayoverse/arsenal/issues
VignetteBuilder: knitr
License: GPL (>= 2)
LazyData: true
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-02 02:15:16 UTC; m144326
Author: Ethan Heinzen [aut, cre],
  Jason Sinnwell [aut],
  Elizabeth Atkinson [aut],
  Tina Gunderson [aut],
  Gregory Dougherty [aut],
  Patrick Votruba [ctb],
  Ryan Lennon [ctb],
  Andrew Hanson [ctb],
  Krista Goergen [ctb],
  Emily Lundt [ctb],
  Brendan Broderick [ctb],
  Maddie McCullough [art]
Maintainer: Ethan Heinzen <heinzen.ethan@mayo.edu>
Repository: CRAN
Date/Publication: 2026-07-02 22:30:19 UTC
