# Generated by roxygen2: do not edit by hand

S3method("[",arsenal_table)
S3method("[",keep_labels)
S3method("[",keep_tableby_attrs)
S3method("[",selectall)
S3method("[",tableby)
S3method("[",tbstat)
S3method("[",tbstat_countpct)
S3method("[",tbstat_multirow)
S3method("[<-",keep_labels)
S3method("[[",tbsta_multirowt)
S3method("[[",tbstat)
S3method("[[",tbstat_countpct)
S3method("labels<-",arsenal_table)
S3method("labels<-",data.frame)
S3method("labels<-",default)
S3method("labels<-",keep_labels)
S3method(Ops,tableby)
S3method(as.data.frame,freqlist)
S3method(as.data.frame,modelsum)
S3method(as.data.frame,summary.freqlist)
S3method(as.data.frame,summary.modelsum)
S3method(as.data.frame,summary.tableby)
S3method(as.data.frame,tableby)
S3method(as.matrix,selectall)
S3method(c,yaml)
S3method(diffs,comparedf)
S3method(diffs,summary.comparedf)
S3method(format,tbstat)
S3method(freqlist,formula)
S3method(freqlist,table)
S3method(head,summary.freqlist)
S3method(head,tableby)
S3method(includeNA,default)
S3method(includeNA,factor)
S3method(is.na,selectall)
S3method(keep.labels,data.frame)
S3method(keep.labels,default)
S3method(labels,arsenal_table)
S3method(labels,data.frame)
S3method(labels,keep_labels)
S3method(loosen.labels,data.frame)
S3method(loosen.labels,default)
S3method(merge,arsenal_table)
S3method(merge,freqlist)
S3method(n.diff.obs,comparedf)
S3method(n.diff.obs,summary.comparedf)
S3method(n.diffs,comparedf)
S3method(n.diffs,summary.comparedf)
S3method(padjust,default)
S3method(padjust,summary.tableby)
S3method(padjust,tableby)
S3method(print,arsenal_table)
S3method(print,code.chunk)
S3method(print,comparedf)
S3method(print,comparedf.frame.summary)
S3method(print,comparedf.vars.summary)
S3method(print,summary.arsenal_table)
S3method(print,summary.comparedf)
S3method(print,tbstat)
S3method(print,verbatim)
S3method(print,yaml)
S3method(sort,freqlist)
S3method(sort,tableby)
S3method(summary,comparedf)
S3method(summary,freqlist)
S3method(summary,modelsum)
S3method(summary,tableby)
S3method(tail,summary.freqlist)
S3method(tail,tableby)
S3method(tests,tableby)
S3method(write2,arsenal_table)
S3method(write2,character)
S3method(write2,code.chunk)
S3method(write2,comparedf)
S3method(write2,default)
S3method(write2,knitr_kable)
S3method(write2,list)
S3method(write2,summary.arsenal_table)
S3method(write2,summary.comparedf)
S3method(write2,verbatim)
S3method(write2,xtable)
S3method(write2,yaml)
S3method(xtfrm,tableby)
export("%nin%")
export("labels<-")
export(Date.mdy)
export(N)
export(Nevents)
export(NeventsSurv)
export(Nmiss)
export(Nmiss2)
export(Nmisspct)
export(Nmisspct2)
export(Npct)
export(Nrisk)
export(NriskSurv)
export(Nrowpct)
export(Nsigntest)
export(allNA)
export(as.selectall)
export(as.tbstat)
export(as.tbstat_multirow)
export(binomCI)
export(clog)
export(code.chunk)
export(comparedf)
export(comparedf.control)
export(countN)
export(countcellpct)
export(countpct)
export(countrowpct)
export(diffs)
export(formulize)
export(freq.control)
export(freqlist)
export(gmean)
export(gmeanCI)
export(gmeansd)
export(gsd)
export(has_strata)
export(includeNA)
export(iqr)
export(is.Date)
export(is.freqlist)
export(is.modelsum)
export(is.selectall)
export(is.summary.freqlist)
export(is.summary.modelsum)
export(is.summary.tableby)
export(is.tableby)
export(is.yaml)
export(keep.labels)
export(loosen.labels)
export(mdy.Date)
export(meanCI)
export(meanpmsd)
export(meanpmse)
export(meansd)
export(meanse)
export(medSurv)
export(medSurvCI)
export(medSurvQuant)
export(medTime)
export(medianmad)
export(medianq1q3)
export(medianrange)
export(modelsum)
export(modelsum.control)
export(modpval.tableby)
export(muck_up_mockstudy)
export(n.diff.obs)
export(n.diffs)
export(na.modelsum)
export(na.paired)
export(na.tableby)
export(negbin)
export(ordinal)
export(padjust)
export(paired)
export(paired.control)
export(pct)
export(q1q3)
export(relrisk)
export(replace2)
export(rowbinomCI)
export(rowpct)
export(selectall)
export(set_attr)
export(set_labels)
export(smart.split)
export(survival)
export(tableby)
export(tableby.control)
export(tbfmt)
export(tests)
export(tol.NA)
export(tol.char.both)
export(tol.char.case)
export(tol.char.none)
export(tol.char.trim)
export(tol.date.absolute)
export(tol.factor.labels)
export(tol.factor.levels)
export(tol.factor.none)
export(tol.logical.none)
export(tol.num.absolute)
export(tol.num.pct)
export(tol.num.percent)
export(tol.other.none)
export(verbatim)
export(write2)
export(write2html)
export(write2pdf)
export(write2word)
export(yaml)
importFrom(utils,head)
importFrom(utils,tail)
