

magic8 <- function(question=NULL){
  sample(c("There is an 'apply' function for that",
         'It is decidedly so',
         'Without a doubt',
         "Need to install another bioconductor package for that",
         'Better off asking Siri',
         'You may rely on it',
         'As I see it, yes',
         'Most likely',
         "It might be time for a walking break",
         'Just bootstrap it',
         'May need to perform simulations',
         'Signs point to yes',
         "You need to go home and rethink your life",
         'Reply hazy; try again',
         'Insufficient sample size',
         'The answer is 7',
         "Bazinga!",
         'Cannot predict right now',
         'Concentrate and ask again',
         "Don't count on it",
         'Maybe, p-value 0.06',
         'Hierarchical clustering always gives an answer',
         'My sources say no',
         'Make a 3-D pie chart',
         "Yes, if the figure is colorful",
         'Very doubtful',
         "PC Load Letter",
         "Not Sure",
         "The possibility of successfully answering your question is approximately three thousand seven hundred and twenty to one!",
         "Depends...did you want the two-tailed probability or the one-tailed?",
         "Error: 404 Not Found",
         "You index is off by one (we won't tell)",
         "Correlation does not equal causation"), 1)
}
