---
title: Test title
---



```

Call:
lm(formula = age ~ sex, data = mockstudy)

Residuals:
    Min      1Q  Median      3Q     Max 
-41.455  -7.455   0.753   8.545  28.753 

Coefficients:
            Estimate Std. Error t value Pr(>|t|)
(Intercept)  60.4552     0.3802 159.001   <2e-16
sexFemale    -1.2082     0.6097  -1.982   0.0477

Residual standard error: 11.51 on 1497 degrees of freedom
Multiple R-squared:  0.002617,	Adjusted R-squared:  0.00195 
F-statistic: 3.927 on 1 and 1497 DF,  p-value: 0.04769


```



