% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.in.R
\name{\%nin\%}
\alias{\%nin\%}
\alias{nin}
\title{Not in}
\usage{
x \%nin\% table
}
\arguments{
\item{x}{vector or \code{NULL}: the values to be matched.}

\item{table}{vector or \code{NULL}: the values to be matched against.}
}
\value{
The negation of \code{\link[base:match]{\%in\%}}.
}
\description{
The not-in operator for R.
}
\examples{
1 \%nin\% 2:10
c("a", "b") \%nin\% c("a", "c", "d")
}
\seealso{
\code{\link[base:match]{\%in\%}}
}
\author{
Raymond Moore
}
