% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.modelsum.R
\name{as.data.frame.modelsum}
\alias{as.data.frame.modelsum}
\title{as.data.frame.modelsum}
\usage{
\method{as.data.frame}{modelsum}(x, ..., labelTranslations = NULL, list.ok = FALSE)
}
\arguments{
\item{x}{A \code{\link{modelsum}} object.}

\item{...}{Arguments to pass to \code{\link{modelsum.control}}.}

\item{labelTranslations}{A named list (or vector) where the name is the label in the
output to be replaced in the pretty rendering by the character string
value for the named element of the list, e.g., \code{list(age = "Age(Years)", meansd = "Mean(SD)")}.}

\item{list.ok}{If the object has multiple by-variables, is it okay to return a list of data.frames instead of a single data.frame?
If \code{FALSE} but there are multiple by-variables, a warning is issued.}
}
\value{
A \code{data.frame}.
}
\description{
Coerce a \code{\link{modelsum}} object to a \code{data.frame}.
}
\seealso{
\code{\link{modelsum}}, \code{\link{summary.modelsum}}
}
\author{
Ethan Heinzen, based on code originally by Greg Dougherty
}
